/* global BDF,HTMLInputElement,HTMLSelectElement,HTMLTextAreaElement */
/**
 * Objet global définissant l'espace de nom Ficheform
 * 
 * @namespace Ficheform
 */
var Ficheform = {};

Ficheform.ARGS = {
  noTab: false  
};
Ficheform.JsManagerInclude = ["_ @title_corpus@", "_ @title_thesaurus@"];
Ficheform.hasChanges = false;
Ficheform.onSubmit = false;

Ficheform.init = function () {
    BDF.$({ficheformRole: "submit-button"}).prop("disabled", true);
    var $form = BDF.$({ficheformRole: "form"}).on('submit', function () {
        Ficheform.onSubmit = true;
    });
    BDF.initBeforeUnload({
        testFunction: function () {
            return (Ficheform.hasChanges) && (!Ficheform.onSubmit);
        }
    });
    BDF.startTimer(function () {
        if (Ficheform.hasChanges) {
            return;
        }
        if (BDF.checkFormChange(BDF.$({ficheformRole: "form"}))) {
            Ficheform.signalChange();
        }
    });
    for(let element of $form[0].elements) {
        let focusable = false;
        if (element instanceof HTMLInputElement) {
            switch(element.type) {
                case "radio":
                case "checkbox":
                case "text":
                    focusable = true;
                    
            }
        } else if (element instanceof HTMLSelectElement) {
            focusable = true;
        } else if (element instanceof HTMLTextAreaElement) {
            focusable = true;
        }
        if (focusable) {
            $(element).focus();
            break;
        }
    }
};

Ficheform.getOption = function (name) {
    if (name === "noTab") {
        return Ficheform.ARGS.noTab;
    }
    return false;
};

/**
 * Retourne sous la forme d'un objet JQuery l'entrée correspondant à la clé
 * en argument.
 * 
 * @param {String} key Clé
 * @returns {JQuery} Objet JQuery correspondant
 */
Ficheform.$entry = function (key) {
    return BDF.$({ficheformRole: "entry", ficheformKey: key});
};

Ficheform.signalChange = function () {
    if (!Ficheform.hasChanges) {
        Ficheform.hasChanges = true;
        BDF.$({ficheformRole: "submit-button"}).prop("disabled", false);
        BDF.clearTimer();
    }
};

Ficheform.getPoidsString = function (value) {
    var idx = value.indexOf("<");
    if (idx === -1) return "";
    return value.substring(idx);
};

Ficheform.getPoids = function (value) {
    var idx = value.indexOf("<");
    if (idx === -1) return "";
    var poids = value.substring(idx+1, value.length -1);
    return poids;
};

Ficheform.checkPoids = function (value) {
    value = $.trim(value);
    if (value.length === 0) return 1;
    if (_is_int()) {
        if (value > 0) return value;
        else return false;
    }
    else return false;
    
    
    function _is_int(){
        if((parseFloat(value) == parseInt(value)) && !isNaN(parseInt(value))) {
            return true;
        } else {
            return false;
        }
    }
};

Ficheform.updateCheckBox = function (checkBox, poids) {
    var currentValue = checkBox.value;
    var idx = currentValue.indexOf("<");
    if (idx > -1) {
        currentValue = currentValue.substring(0, idx);
    }
    if (poids > 1) {
        checkBox.value = currentValue + "<" + poids + ">";
    } else  {
        checkBox.value = currentValue;
    }
};

Ficheform.openDialog = function (dialogUrl, dialogName, width, height) {
    var w = window.open(dialogUrl,dialogName,"resizable=yes,width=" + width + ",height=" + height + ",scrollbars");
    w.focus();
};

Ficheform.removeFromValue = function(value, id, separator) {
    var idString = String(id);
    var array = value.split(";");
    var newVal = "";
    for(let token of array) {
        if (separator) {
            let idx = token.indexOf(separator);
            if (idx > 0) {
                let tokenId = token.substring(0, idx);
                if (tokenId !== idString) {
                    newVal += token + ";";
                }
            }
        } else {
            if (token !== idString) {
                newVal += token + ";";
            }
        }
    }
    return newVal;
};

Ficheform.initPoidsInput = function ($entry) {
    var $ficheList = BDF.$children($entry, {ficheformRole: "fiche-list"});
    let buttonId = BDF.generateId();
    $ficheList.append(BDF.render("ficheform:pioche", {
        buttonId: buttonId,
        action: "action-ficheform-Poids_on",
        locKey: "_ link.edition.poids_on",
        noTab: Ficheform.getOption("noTab")
    }));
    $$(buttonId).data("state", "on").click(function () {
       let $this = $(this);
       if ($this.data("state") === "on") {
           $this.removeClass("action-ficheform-Poids_on").addClass("action-ficheform-Poids_off");
           $this.attr("title", BDF.Loc.get("_ link.edition.poids_off"));
           $this.data("state", "off");
           _display();
       } else {
           $this.removeClass("action-ficheform-Poids_off").addClass("action-ficheform-Poids_on");
           $this.attr("title", BDF.Loc.get("_ link.edition.poids_on"));
           $this.data("state", "on");
           _save();
       }
    });
        
        function _display() {
            BDF.$($ficheList, {ficheformRole: "fiche-listitem"}).each(function (index, element) {
                let $div = $(element);
                let $checkbox = BDF.$($div, {ficheformRole: "fiche-checkbox"});
                let poids = Ficheform.getPoids($checkbox[0].value);
                $checkbox.after(BDF.render("ficheform:poidsinput", {
                    poids: poids
                }));
            });
            BDF.$($ficheList, {ficheformRole: "poids"}).hide();
        }

        function _save() {
            BDF.$($ficheList, {ficheformRole: "fiche-listitem"}).each(function (index, element) {
                let $div = $(element);
                let $poidsInput = BDF.$($div, {ficheformRole: "poids-input"})
                let value = $poidsInput[0].value;
                let checkBox = BDF.$($div, {ficheformRole: "fiche-checkbox"})[0];
                if (Ficheform.checkPoids(value)) {
                    Ficheform.updateCheckBox(checkBox, value);
                }
                $poidsInput.remove();
                BDF.$($div, {ficheformRole: "poids"}).text(Ficheform.getPoidsString(checkBox.value)).show();
            });
        }
};
